<?php
/**
 * Created by PhpStorm.
 * User: Mead
 * Date: 2019/9/3
 * Time: 8:05 PM
 */

namespace App\Servers;

/**
 * 登录报文
 * Class LoginServer
 * @package App\Servers
 */
class LoginServer extends BaseServer
{

    public function main($body)
    {
        $data = $this->decode($body);

        self::log($data['box_no'], 'LoginServer', self::$LOG_MAJOR);

        // 判断有没有该车
        $bike_no = $_SESSION['bike_no'];

        if (!$bike_no) {
            return $this->response(self::LOGIN_TYPE_FAIL);
        }

        $cols['is_link'] = \App\Maps\BikeMap::LINK_ONLINE;
        $this->db->update('bikes')->where('box_no = ' . $data['box_no'])->cols($cols)->query();

        // 发送登录成功通知
        return $this->response(self::LOGIN_TYPE_SUCCESS);
    }

    // 登录成功
    const LOGIN_TYPE_SUCCESS = '00';
    // 登录失败
    const LOGIN_TYPE_FAIL = '01';
    // 登录无效
    const LOGIN_TYPE_INVALID = '02';
    // 登录非法
    const LOGIN_TYPE_ILLEGAL = '03';

    /**
     * 登录响应
     * @param $login_type
     * @return array
     * User: Mead
     */
    public function response($login_type = '00')
    {
        $time = time();
        $hex = dechex($time);
        $body = [
            $login_type,
            $hex
        ];
        $body = str_split(implode('', $body), 2);
        return $body;
    }

    /**
     * 解析装载的登录数据
     * @param $body
     * @return array
     * User: Mead
     */
    private function decode($body)
    {
        $i = 0;
        $box_no = self::box_no(self::stitching($body, $i, 5));
        $i += 5;
        $box_time = self::stitching($body, $i, 4);
        $i += 4;
        $imsi = self::stitching($body, $i, 8);
        $i += 8;
        $imei = self::stitching($body, $i, 8);
        $i += 8;
//        $box_function = self::stitching($body, $i, 2);
        $i += 2;
//        $factory_no = self::stitching($body, $i, 8);
        $i += 8;
        $software_version = self::stitching($body, $i, -1);
        return [
            'box_no' => $box_no,
            'box_time' => $this->decodeTime($box_time),
            'imsi' => $imsi,
            'imei' => $imei,
            'software_version' => $software_version
        ];
    }

    /**
     * 处理设备序列号
     * @param $no
     * @return bool|string
     * User: Mead
     */
    private static function box_no($no)
    {
        return substr($no, 0, 9);
    }
}